function [ alpha ] = KLiMapS(s, D, k, DINV, maxIter)
%
% [ alpha ] = KLIMAPS(s, D, DINV, gamma)
%
%   Returns the k-sparse vector alpha which satisfies underdetermined 
%   system of equations  D*alpha=s, using  Lipschitzian Mapping for Sparsity
%   Recovery algorithm. 
%   The dictionary matrix  D should have more columns than rows. The 
%   number of rows of dictionary D  must be equal to the length of the 
%   column observation vector(or matrix) s.
%
%     The observations signal(s) s and the dictionary D necessarily, must 
%   be provided by the user. The other parameters have defult values, or
%   may be provided by the user.
% 
%   This algorithm work fine with matrix too, returning the sparsest matrix
%   alpha that satisfy the underdetermined system of equations D*alpha=s.
%   Then the dictionary D is of size [n x m], s must be of size [ s x T ] and
%   as consiquence the coefficients matirx alpha is of size [ m x T]
% 
%   s: 
%       The observation vector (matrix)
%        
%   D: 
%       The dictionary 
%   k:
%       Number of coefficients to select
%   DINV (optional): 
%       The pseudo-inverse of the dictionary matrix D
%
%   gamma (optional): 
%       The increase factor (default 1.01)
%
%   maxIter (optional): 
%       Maximum number of iterations (default 1000)
%
% Authors: Alessandro Adamo and Giuliano Grossi
% Version: 1.0
% Last modified: 1 Aug. 2011.
%
% WEB PAGE:
% ------------------
%    http://dalab.dsi.unimi.it/klimaps
%
% HISTORY:
%--------------
% Version 1.0: 1 Aug. 2011
%        first official version.
%
    if(nargin<3||nargin>5)
        error('Wrong parameters number');
    end
    if(nargin==3)
        DINV=pinv(D);
        maxIter=1000;
    end
    if(nargin==4)
        maxIter=1000;
    end
    
    %% --  INITIALIZATION  --
    alpha = DINV*s;
    a=sort(abs(alpha));
    lambda = 1/a(end-k);

    epsilon=1e-5;                    % stopping criteria
   
    %% --  CORE  --
    for i=1:maxIter

        alphaold=alpha;

        % apply sparsity constraction mapping: increase sparsity
        beta = alpha.*(1-exp(-lambda.*abs(alpha)));

        % apply the orthogonal projection
        alpha = beta-DINV*(D*beta-s);       
        % update the lambda coefficient
        a=sort(abs(alpha));
        lambda = 1/a(end-k);
        
        % check the stopping criteria
        if (norm(alpha-alphaold)<epsilon || isnan(lambda))
            break;
        end

    end

    %% -- REFINEMENT --

    % final refinements of the solution
    alpha(abs(alpha)<=1/lambda)=0;
    
    idx = alpha~=0;
    D1 = D(:,idx);
    alpha(idx) = alpha(idx) - pinv(D1)*(D1*alpha(idx)-s);
  
    
end



